*!	-----------------------------------------------------------------
*!
*!	mbuilder.prg
*!	Main module for MasterBuilder 2.0
*!
*!	Copyright (c) 1995-1996 Colin Nicholls
*!	All Rights Reserved	
*!
*!	Notes
*!	-----
*!	See README.TXT for documentation and full revision history	
*!
*!	-----------------------------------------------------------------

#include BUILDER.H

parameters param1, param2, param3, param4

*//		Save the current settings:

local llTalkON, llExactON
if set('TALK')='ON'
	llTalkON = .T.
	set talk off
else
	llTalkON = .F.
endif
if set('EXACT')='ON'
	llExactON = .T.
else
	set exact on
	llExactON = .F.
endif	

*//		Store the current program:

local llContinue, loObject, lcHowCalled, llMasterBuilder

llContinue = l_YES

private gcSelf
gcSelf	 = sys(16)


do case
case parameters() = 0

	*//		If mbuilder was called with no parameters, then
	*//		register the builder:

	do CheckRegister
	llContinue = l_NO


case type( 'm.param1' ) = c_TYPE_NUMERIC

	*//		General purpose switch to get at forms and 
	*//		programs inside the app. 

	*//		This is horrible, but in a modeless environment 
	*//		I don't see another way to get to my sub-forms
	*//		and sub-procedures that are inside the app.

	local llAlwaysOnTop
	do case
	case m.param1 = n_MBUILDER_ERROR
	
		*//		Error condition:
		
		do merror with m.param2, m.param3, m.param4

	case m.param1 = n_MBUILDER_TABLE

		OpenBuilderTable( m.param2 )
	
	case m.param1 = n_MBUILDER_INFO
	
		*//		About / More info box:
		
		llAlwaysOnTop = _MASTERBUILDER.AlwaysOnTop
		_MASTERBUILDER.lLockBuilder = l_YES
		_MASTERBUILDER.AlwaysOnTop	= l_NO
		do form about with m.param2
		_MASTERBUILDER.AlwaysOnTop	= m.llAlwaysOnTop

	case m.param1 = n_MBUILDER_OPTIONS
	
		*//		Options dialog:
		
		llAlwaysOnTop = _MASTERBUILDER.AlwaysOnTop
		_MASTERBUILDER.lLockBuilder = l_YES
		_MASTERBUILDER.AlwaysOnTop	= l_NO
		do form options
		_MASTERBUILDER.AlwaysOnTop	= m.llAlwaysOnTop

	case m.param1 = n_MBUILDER_CHANGECLASS
	
		*//		Change Class dialog:
		
		llAlwaysOnTop = _MASTERBUILDER.AlwaysOnTop
		_MASTERBUILDER.lLockBuilder = l_YES
		_MASTERBUILDER.AlwaysOnTop	= l_NO
		do form ReClass with m.param2
		_MASTERBUILDER.AlwaysOnTop	= m.llAlwaysOnTop
	
	case m.param1 = n_MBUILDER_LOADLIBRARY
	
		*//		Load class library:
		
		set classlib to mbuilder additive

	case m.param1 = n_MBUILDER_FONTS
	
		do form fonts with m.param2

	endcase

	if llTalkON
		set talk on
	endif
	if not llExactON
		set exact off
	endif
	
	return

		
case type( 'm.param1' ) = c_TYPE_CHARACTER

	*//		We have been called from the Registry. We
	*//		must obtain our parameters from the variables
	*//		set up by BUILDER.APP:

	external array wbaControl

	if type( 'wbaControl[1]' ) = c_TYPE_OBJECT

		loObject 		= wbaControl[1]
		lcHowCalled 	= m.wbcpOrigin
		llMasterBuilder = l_FALSE

		if loObject.Name = "Screen"
			=MessageBox( "MasterBuilder was not designed to operate on the Screen object. You must be in design mode to use MasterBuilder.",MB_OK+MB_ICONINFORMATION, c_TITLE) 
			llContinue = l_NO
		endif
	else
		=MessageBox( "MasterBuilder has been called with invalid parameters!",MB_OK+MB_ICONEXCLAMATION, c_TITLE) 
		llContinue = l_NO
	endif
	
case type( 'm.param1' ) = c_TYPE_OBJECT

	*//		We have been called via _BUILDER. We must
	*//		use the parameters passed:

	loObject 		= m.param1
	lcHowCalled 	= m.param2
	llMasterBuilder = l_TRUE

otherwise

	=MessageBox( "MasterBuilder has been called with invalid parameters!",MB_OK+MB_ICONEXCLAMATION, c_TITLE) 
	llContinue = l_NO
	
endcase


if llContinue

	*//		open the builder table
	
	OpenBuilderTable( n_READONLY )

	*//		Was more than one object selected?

	local lnObjectsSelected
	private paoGroup

	lnObjectsSelected = aselobj( paoGroup )

	if lnObjectsSelected = 0
		
		*//		For example, rightclicking on a form:
		
		lnObjectsSelected = 1
		dimension paoGroup[1]
		paoGroup[1] = loObject

	endif

	*//		Now we have an array of objects, possibly only one.
	*//		We can pass this information on to the MBuilder form:


	*//		Is mbuilder already on the desktop?

	if  type('_MASTERBUILDER') = c_TYPE_UNDEFINED ;
	or (type('_MASTERBUILDER') = c_TYPE_OBJECT and isnull(_MASTERBUILDER))

		*//		First time: create

		set classlib to mbuilder additive

		public _MASTERBUILDER

		_MASTERBUILDER = createobject( 'kfrmBuilder' )

		if type('_MASTERBUILDER') <> c_TYPE_OBJECT or isnull(_MASTERBUILDER)
			*//		The init didn't work.
			release _MASTERBUILDER
			return
			
		endif

		release classlib mbuilder

	endif

	*//		update MasterBuilder properties

	with _MASTERBUILDER
		.oRef			= loObject
		.cSelf		  	= gcSelf
		.cHowCalled		= lcHowCalled
		.Name 		  	= "MasterBuilder"
	endwith

	*//		Display the form: (The .Activate method 
	*//		will update the interal aObjects[] array)

	_MASTERBUILDER.Show( n_MODELESS )

endif

if llTalkON
	set talk on
endif
if not llExactON
	set exact off
endif

return


procedure OpenBuilderTable( tnMode )
	*//		Open the modules database:
	*//		Two modes: 
	*//			1	Read/Only
	*//			2   Read/Write
	*//
	if not used('mbuilder')
		local lcBuilderPath
		lcBuilderPath = left(m.gcSelf,rat("\",m.gcSelf))		
		if file( m.lcBuilderPath + "MBUILDER.DBF" )
			do case
			case m.tnMode = n_READWRITE
				use (m.lcBuilderPath + "MBUILDER.DBF") in 0 

			case m.tnMode = n_READONLY
				use (m.lcBuilderPath + "MBUILDER.DBF") in 0 noupdate
			
			otherwise
				use (m.lcBuilderPath + "MBUILDER.DBF") in 0 noupdate
			
			endcase
		else
		
			*//		Create it:
			
			create table (m.lcBuilderPath + "MBUILDER.DBF") ;
				( BASECLASS c(30), CLASS c(30), PROGRAM c(40) )
		
			use in mbuilder
		
			*//		Now open it again:
		
			OpenBuilderTable( m.tnMode )
			
		endif
	endif
	return

endproc


procedure CheckRegister

	local lnSelect, lcRegistry, lcSelf
	lnSelect   = select()
	lcRegistry = m_BUILDER_REGISTRY

	if MessageBox( "You can not run the MasterBuilder directly. Would you like to register it in the BUILDER registry?",MB_YESNO+MB_ICONQUESTION, c_TITLE) = IDNO
		return
	endif

	*//		look for the registry table:
	
	if file( m.lcRegistry  )

		select 0
		use ( m.lcRegistry ) again shared alias bReg

	else
		*//		if you can't find the registry table,
		*//		ask the user for it:
		
		if MessageBox( "I can not find BUILDER.DBF. Would you like to locate it?",MB_YESNO+MB_ICONQUESTION, c_TITLE) = IDNO
			return
		else
			lcRegistry = getfile("DBF","Where is BUILDER.DBF?")
			if not empty( m.lcRegistry )
				select 0
				use (m.lcRegistry) again shared alias bReg
			else
				return
			endif
		endif	
	endif
		
	*//  once you've found the registry table, ensure
	*//  that the builder is registered 
	
	*//  locate for the right record:
	locate for NAME = c_REG_NAME
	if found()
		replace DESCRIPT with c_REG_DESCRIPT, ;
				TYPE     with "ALL",;
				PROGRAM  with m.gcSelf
	else	
		*//		register the builder:
		insert into bReg ;
			( NAME, DESCRIPT, TYPE, PROGRAM ) ;
			values ( c_REG_NAME, c_REG_DESCRIPT, "ALL", m.gcSelf )
	endif
	*// close the register:

	use in bReg
	select (m.lnSelect)	

	*// ask if you want to set _BUILDER:
	
	if MessageBox( "Master Builder is registered. Do you want to set _BUILDER to this program?",MB_YESNO+MB_ICONQUESTION, c_TITLE) = IDYES

		_BUILDER = m.gcSelf

	endif
	return
	
endproc
	

